import { prefixes as knownPrefixes, shrink } from '@zazuko/rdf-vocabularies';
function prefixDeclarations(prefixes) {
    return [...prefixes]
        .filter(prefix => prefix in knownPrefixes)
        .map(prefix => `PREFIX ${prefix}: <${knownPrefixes[prefix]}>`)
        .join('\n');
}
export class SparqlTemplateResult {
    constructor(strings, values) {
        this.prefixes = new Set();
        this.strings = strings;
        this.values = values;
    }
    toString(options = {}) {
        const prefixes = options.prefixes || typeof options.prefixes === 'undefined';
        const baseRegex = new RegExp('^' + options.base);
        const l = this.strings.length - 1;
        let query = '';
        for (let i = 0; i < l; i++) {
            const s = this.strings[i];
            const value = this.values[i];
            let valueStr;
            if (typeof value === 'string') {
                valueStr = `"${value}"`;
            }
            else if (typeof value === 'undefined' || value === null) {
                valueStr = '';
            }
            else if (value instanceof SparqlTemplateResult) {
                valueStr = value.toString({ prefixes: false });
                value.prefixes.forEach(prefix => this.prefixes.add(prefix));
            }
            else {
                switch (value.termType) {
                    case 'Literal':
                        valueStr = `"${value.value}"`;
                        break;
                    case 'NamedNode':
                        {
                            const shrunk = shrink(value.value);
                            if (shrunk) {
                                this.prefixes.add(shrunk.split(':')[0]);
                                valueStr = shrunk;
                            }
                            else {
                                valueStr = `<${value.value.replace(baseRegex, '')}>`;
                            }
                        }
                        break;
                    case 'Variable':
                        valueStr = `?${value.value}`;
                        break;
                    default:
                        valueStr = value.value;
                }
            }
            query += s + valueStr;
        }
        query += this.strings[l];
        let baseStatement = '';
        if (options.base) {
            baseStatement = `BASE <${options.base}>\n\n`;
        }
        let prefixStatements = '';
        if (prefixes && this.prefixes.size > 0) {
            prefixStatements = `${prefixDeclarations(this.prefixes)}\n\n`;
        }
        return `${baseStatement}${prefixStatements}${query}`;
    }
}
export const sparql = (strings, ...values) => new SparqlTemplateResult(strings, values);
